#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc
import itertools
import threading
import time

from oslo_config import cfg


class BaseOvnDbSynchronizer(metaclass=abc.ABCMeta):

    _required_mechanism_drivers = []
    _required_service_plugins = []
    _required_ml2_ext_drivers = []

    def __init__(self, core_plugin, ovn_driver, mode, is_maintenance=False,
                 plugin_conf=None):
        """Initialize the OVN DB synchronizer.

        :param core_plugin: The Neutron core plugin instance
        :param ovn_driver: The OVN driver instance containing the OVN NB and SB
                           API clients
        :param mode: The synchronization mode, valid values are:
                     'off', 'log', 'repair' or 'migrate'
        :param is_maintenance: Flag indicating if running in maintenance mode.
                               Defaults to False
        :param plugin_conf: Optional isolated oslo.config object with
                            plugin-specific configuration
        """
        self.core_plugin = core_plugin
        self.ovn_nb_api = ovn_driver.nb_ovn
        self.ovn_sb_api = ovn_driver.sb_ovn
        self.ovn_driver = ovn_driver
        self.mode = mode
        self.is_maintenance = is_maintenance
        self.plugin_conf = plugin_conf
        self._thread = None

    def sync(self, delay_seconds=10):
        time.sleep(delay_seconds)
        self._thread = threading.Thread(target=self.do_sync)
        self._thread.start()

    @classmethod
    def _get_parent_class_attribute(cls, cls_attr):
        parent_attibutes = set()
        for parent in cls.__mro__:
            parent_attibutes |= set(getattr(parent, cls_attr, set()))
        return list(parent_attibutes | set(getattr(cls, cls_attr, [])))

    @classmethod
    def get_required_mechanism_drivers(cls):
        return cls._get_parent_class_attribute('_required_mechanism_drivers')

    @classmethod
    def get_required_service_plugins(cls):
        return cls._get_parent_class_attribute('_required_service_plugins')

    @classmethod
    def get_required_ml2_extension_drivers(cls):
        return cls._get_parent_class_attribute('_required_ml2_ext_drivers')

    @classmethod
    def prepare_additional_configuration(cls, conf):
        """Do any additional configuration changes required by plugin.

        :param: conf: The oslo_config.cfg.CONF object
        """

    @classmethod
    def register_additional_cli_opts(cls, conf):
        """Register plugin-specific CLI opts on global CONF before parsing."""

    @classmethod
    def register_plugin_config_opts(cls, conf):
        """Register plugin config opts on an isolated ConfigOpts object."""

    @classmethod
    def get_plugin_config_files(cls, global_conf):
        """Return list of config file paths from CLI, or [] if none."""
        return []

    @classmethod
    def load_plugin_configuration(cls, global_conf):
        """Parse plugin config files into isolated ConfigOpts.

        :returns: cfg.ConfigOpts instance or None if no config files specified
        """
        config_files = cls.get_plugin_config_files(global_conf)
        if not config_files:
            return None
        plugin_conf = cfg.ConfigOpts()
        cls.register_plugin_config_opts(plugin_conf)
        args = list(itertools.chain.from_iterable(
            ['--config-file', f] for f in config_files))
        plugin_conf(args)
        return plugin_conf

    @abc.abstractmethod
    def do_sync(self):
        """Method to sync the OVN DB."""

    def stop(self):
        # TODO(ralonsoh): this method and the function executed in the thread
        # should support a fast exit way.
        try:
            self._thread.join()
        except AttributeError:
            # Haven't started syncing
            pass



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# This role is used only for communication between services, it shouldn't be
# used by human users
SERVICE = 'rule:service_api'

# For completion of the phase 1
# https://governance.openstack.org/tc/goals/selected/consistent-and-secure-rbac.html#phase-1
# there is now ADMIN role
ADMIN = "rule:admin_only"

# This check string is the primary use case for the project's manager who is
# more privileged user then typical MEMBER of the project.
PROJECT_MANAGER = 'role:manager and project_id:%(project_id)s'

# This check string is the primary use case for typical end-users, who are
# working with resources that belong to a project (e.g., creating ports and
# routers).
PROJECT_MEMBER = 'role:member and project_id:%(project_id)s'

# This check string should only be used to protect read-only project-specific
# resources. It should not be used to protect APIs that make writable changes
# (e.g., updating a router or deleting a port).
PROJECT_READER = 'role:reader and project_id:%(project_id)s'

# The following are common composite check strings that are useful for
# protecting APIs designed to operate with multiple scopes (e.g.,
# an administrator should be able to delete any router in the deployment, a
# project member should only be able to delete routers in their project).
ADMIN_OR_SERVICE = (
    '(' + ADMIN + ') or (' + SERVICE + ')')
ADMIN_OR_PROJECT_MANAGER = (
    '(' + ADMIN + ') or (' + PROJECT_MANAGER + ')')
ADMIN_OR_PROJECT_MEMBER = (
    '(' + ADMIN + ') or (' + PROJECT_MEMBER + ')')
ADMIN_OR_PROJECT_READER = (
    '(' + ADMIN + ') or (' + PROJECT_READER + ')')

# In some cases we need to check owner of the parent resource, it's like that
# for example for QoS rules (check owner of QoS policy rule belongs to) or
# Floating IP port forwarding (check owner of FIP which PF is using). It's like
# that becasue those resources (QOS rules, FIP PFs) don't have project_id
# attribute at all and they belongs to the same project as parent resource (QoS
# policy, FIP).
RULE_PARENT_OWNER = 'rule:ext_parent_owner'
PARENT_OWNER_MANAGER = 'role:manager and ' + RULE_PARENT_OWNER
PARENT_OWNER_MEMBER = 'role:member and ' + RULE_PARENT_OWNER
PARENT_OWNER_READER = 'role:reader and ' + RULE_PARENT_OWNER
ADMIN_OR_PARENT_OWNER_MANAGER = (
    '(' + ADMIN + ') or (' + PARENT_OWNER_MANAGER + ')')
ADMIN_OR_PARENT_OWNER_MEMBER = (
    '(' + ADMIN + ') or (' + PARENT_OWNER_MEMBER + ')')
ADMIN_OR_PARENT_OWNER_READER = (
    '(' + ADMIN + ') or (' + PARENT_OWNER_READER + ')')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from neutron_lib.api.definitions import pvlan
from neutron_lib.services.pvlan import constants as pvlan_const
from neutron_lib.tests.unit.api.definitions import base


class PVLANDefinitionTestCase(base.DefinitionBaseTestCase):

    extension_module = pvlan
    extension_resources = ()
    extension_attributes = (pvlan_const.PVLAN,
                            pvlan_const.PVLAN_TYPE,
                            pvlan_const.PVLAN_COMMUNITY,)



# Copyright (c) 2021 Ericsson Software Technology
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api import converters
from neutron_lib.api.definitions import qos as qos_apidef
from neutron_lib import constants
from neutron_lib.db import constants as db_const
from neutron_lib.services.qos import constants as qos_constants
from neutron_lib.types import (
    ActionMap,
    ResourceAttributeMap,
    SubResourceAttributeMap,
)

ALIAS = 'qos-pps-minimum'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
RESOURCE_NAME = 'minimum_packet_rate_rule'
COLLECTION_NAME = RESOURCE_NAME + 's'
NAME = 'QoS minimum packet rate rule'
API_PREFIX = '/' + qos_apidef.ALIAS
DESCRIPTION = "Allow to configure QoS minimum packet rate rule."
UPDATED_TIMESTAMP = '2021-07-14T10:00:00-00:00'
RESOURCE_ATTRIBUTE_MAP: ResourceAttributeMap = {}
SUB_RESOURCE_ATTRIBUTE_MAP: SubResourceAttributeMap = {
    COLLECTION_NAME: {
        'parent': qos_apidef._PARENT,
        'parameters': {
            **qos_apidef._QOS_RULE_COMMON_FIELDS,
            qos_constants.MIN_KPPS: {
                'allow_post': True,
                'allow_put': True,
                'convert_to': converters.convert_to_int,
                'is_visible': True,
                'is_filter': True,
                'is_sort_key': True,
                'validate': {
                    'type:range': (0, db_const.DB_INTEGER_MAX_VALUE),
                },
            },
            qos_constants.DIRECTION: {
                'allow_post': True,
                'allow_put': True,
                'is_visible': True,
                'default': constants.EGRESS_DIRECTION,
                'is_filter': True,
                'is_sort_key': True,
                'validate': {
                    'type:values': constants.VALID_DIRECTIONS_AND_ANY,
                }
            }
        }
    }
}

ACTION_MAP: ActionMap = {}
REQUIRED_EXTENSIONS = [qos_apidef.ALIAS]
OPTIONAL_EXTENSIONS = []
ACTION_STATUS = {}



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.db import constants as db_const
from neutron_lib.types import (
    ActionMap,
    ResourceAttributeMap,
    SubResourceAttributeMap,
)

ALIAS = 'security-groups-name-restrictions'

IS_SHIM_EXTENSION = False

IS_STANDARD_ATTR_EXTENSION = False

NAME = 'Security group name restrictions'
DESCRIPTION = (
    'Enforce printable characters and no leading/trailing whitespace '
    'in security group names, consistent with other Neutron resources'
)

UPDATED_TIMESTAMP = '2026-07-29T10:00:00-00:00'

COLLECTION_NAME = 'security_groups'

RESOURCE_ATTRIBUTE_MAP: ResourceAttributeMap = {
    COLLECTION_NAME: {
        'name': {
            'allow_post': True,
            'allow_put': True,
            'is_visible': True,
            'default': '',
            'is_filter': True,
            'is_sort_key': True,
            'validate': {
                'type:name_string_not_default': db_const.NAME_FIELD_SIZE},
        },
    },
}

SUB_RESOURCE_ATTRIBUTE_MAP: SubResourceAttributeMap = {}

ACTION_MAP: ActionMap = {}

ACTION_STATUS = {}

REQUIRED_EXTENSIONS = [
    'security-group',
]

OPTIONAL_EXTENSIONS = []



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from neutron_lib.api import converters
from neutron_lib.api.definitions import stateful_security_group
from neutron_lib.db import constants as db_const
from neutron_lib.types import (
    ActionMap,
    ResourceAttributeMap,
    SubResourceAttributeMap,
)

ALIAS = 'security-groups-default-statefulness'

IS_SHIM_EXTENSION = False

IS_STANDARD_ATTR_EXTENSION = False

NAME = 'Security Groups Default Statefulness'

DESCRIPTION = (
    "Allows configuring the default value of the 'stateful' attribute "
    "for new security groups, on a per-project or system-wide basis")

UPDATED_TIMESTAMP = '2026-04-13T00:00:00-00:00'

RESOURCE_NAME = 'security_groups_default_statefulness'
COLLECTION_NAME = 'security_groups_default_statefulness'

ID = 'id'
PROJECT_ID = 'project_id'
STATEFUL = 'stateful'

RESOURCE_ATTRIBUTE_MAP: ResourceAttributeMap = {
    COLLECTION_NAME: {
        ID: {'allow_post': False,
             'allow_put': False,
             'validate': {'type:uuid': None},
             'is_visible': True,
             'primary_key': True,
             'is_sort_key': True,
             'is_filter': True},
        PROJECT_ID: {
            'allow_post': True,
            'allow_put': False,
            'validate': {
                'type:string_or_none':
                    db_const.PROJECT_ID_FIELD_SIZE},
            'is_visible': True,
            'required_by_policy': True,
            'default': None,
            'is_sort_key': True,
            'is_filter': True},
        STATEFUL: {
            'allow_post': True,
            'allow_put': True,
            'convert_to': converters.convert_to_boolean,
            'default': True,
            'is_visible': True,
            'is_filter': True,
            'is_sort_key': True},
    }
}

SUB_RESOURCE_ATTRIBUTE_MAP: SubResourceAttributeMap = {}

ACTION_MAP: ActionMap = {}

ACTION_STATUS = {
}

REQUIRED_EXTENSIONS = [
    'security-group',
    stateful_security_group.ALIAS,
]

OPTIONAL_EXTENSIONS = [
]



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api import converters
from neutron_lib.api.definitions import agent
from neutron_lib import constants
from neutron_lib.types import (
    ActionMap,
    ResourceAttributeMap,
    SubResourceAttributeMap,
)


ALIAS = 'l3-agent-scheduler-ha-chassis-priority'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'L3 Agent Scheduler HA Chassis Priority'
API_PREFIX = ''
DESCRIPTION = ('Exposes the HA_Chassis priority in the L3 Agent Scheduler '
               'API responses and allows setting priority when scheduling '
               'a router to a chassis.')
UPDATED_TIMESTAMP = '2026-03-31T00:00:00-00:00'
RESOURCE_NAME = agent.RESOURCE_NAME
COLLECTION_NAME = agent.COLLECTION_NAME
HA_CHASSIS_PRIORITY = 'ha_chassis_priority'

RESOURCE_ATTRIBUTE_MAP: ResourceAttributeMap = {
    COLLECTION_NAME: {
        HA_CHASSIS_PRIORITY: {
            'allow_post': True,
            'allow_put': True,
            'default': constants.ATTR_NOT_SPECIFIED,
            'is_visible': True,
            'convert_to': converters.convert_to_int_if_not_none,
            'enforce_policy': True,
        }
    }
}
SUB_RESOURCE_ATTRIBUTE_MAP: SubResourceAttributeMap = {}
ACTION_MAP: ActionMap = {}
ACTION_STATUS = {}
REQUIRED_EXTENSIONS = [
    agent.ALIAS,
    constants.L3_AGENT_SCHEDULER_EXT_ALIAS,
]
OPTIONAL_EXTENSIONS = []



#
# Copyright 2017 Ericsson India Global Services Pvt Ltd. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#

from neutron_lib.api import converters as lib_converters
from neutron_lib.api.definitions import bgpvpn
from neutron_lib import constants
from neutron_lib.types import (
    ActionMap,
    ResourceAttributeMap,
    SubResourceAttributeMap,
)


VNI = 'vni'

# The alias of the extension.
ALIAS = 'bgpvpn-vni'

# Whether or not this extension is simply signaling behavior to the user
# or it actively modifies the attribute map.
IS_SHIM_EXTENSION = False

# Whether the extension is marking the adoption of standardattr model for
# legacy resources, or introducing new standardattr attributes. False or
# None if the standardattr model is adopted since the introduction of
# resource extension.
# If this is True, the alias for the extension should be prefixed with
# 'standard-attr-'.
IS_STANDARD_ATTR_EXTENSION = False

# The name of the extension.
NAME = 'BGPVPN VXLAN VNI extension'

# A prefix for API resources. An empty prefix means that the API is going
# to be exposed at the v2/ level as any other core resource.
API_PREFIX = ''

# The description of the extension.
DESCRIPTION = "BGPVPN VXLAN VNI extension"

# A timestamp of when the extension was introduced.
UPDATED_TIMESTAMP = "2017-09-12T10:00:00-00:00"


RESOURCE_ATTRIBUTE_MAP: ResourceAttributeMap = {
    bgpvpn.COLLECTION_NAME: {
        VNI: {
            'allow_post': True, 'allow_put': False,
            'convert_to': lib_converters.convert_to_int_if_not_none,
            'default': None,
            'validate': {
                'type:range_or_none': (constants.MIN_VXLAN_VNI,
                                       constants.MAX_VXLAN_VNI),
            },
            'is_visible': True, 'enforce_policy': True},
    },
}

# The subresource attribute map for the extension. It adds child resources
# to main extension's resource. The subresource map must have a parent and
# a parameters entry. If an extension does not need such a map, None can
# be specified (mandatory).
SUB_RESOURCE_ATTRIBUTE_MAP: SubResourceAttributeMap = {}

# The action map: it associates verbs with methods to be performed on
# the API resource.
ACTION_MAP: ActionMap = {}

# The action status.
ACTION_STATUS = {
}

# The list of required extensions.
REQUIRED_EXTENSIONS = [bgpvpn.ALIAS]

# The list of optional extensions.
OPTIONAL_EXTENSIONS = []



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import tempfile
from unittest import mock

from oslo_config import cfg

from neutron_lib.ovn import db_sync
from neutron_lib.tests import _base as base


class TestBaseOvnDbSynchronizer(base.BaseTestCase):

    def setUp(self):
        super().setUp()
        self.core_plugin = mock.Mock()
        self.ovn_driver = mock.Mock()
        self.ovn_driver.nb_ovn = mock.Mock()
        self.ovn_driver.sb_ovn = mock.Mock()

    def _create_synchronizer(self):
        class TestSynchronizer(db_sync.BaseOvnDbSynchronizer):
            def do_sync(self):
                pass

        return TestSynchronizer(
            self.core_plugin, self.ovn_driver, 'repair')

    @mock.patch('neutron_lib.ovn.db_sync.threading.Thread')
    @mock.patch('neutron_lib.ovn.db_sync.time.sleep')
    def test_sync_with_default_delay(self, mock_sleep, mock_thread):
        synchronizer = self._create_synchronizer()
        mock_thread_instance = mock.Mock()
        mock_thread.return_value = mock_thread_instance

        synchronizer.sync()

        # Verify sleep was called with default delay
        mock_sleep.assert_called_once_with(10)
        # Verify thread was created with correct target
        mock_thread.assert_called_once_with(target=synchronizer.do_sync)
        # Verify thread was started
        mock_thread_instance.start.assert_called_once()

    @mock.patch('neutron_lib.ovn.db_sync.threading.Thread')
    @mock.patch('neutron_lib.ovn.db_sync.time.sleep')
    def test_sync_with_custom_delay(self, mock_sleep, mock_thread):
        synchronizer = self._create_synchronizer()
        mock_thread_instance = mock.Mock()
        mock_thread.return_value = mock_thread_instance

        synchronizer.sync(delay_seconds=5)

        # Verify sleep was called with custom delay
        mock_sleep.assert_called_once_with(5)
        mock_thread.assert_called_once_with(target=synchronizer.do_sync)
        mock_thread_instance.start.assert_called_once()

    @mock.patch('neutron_lib.ovn.db_sync.threading.Thread')
    @mock.patch('neutron_lib.ovn.db_sync.time.sleep')
    def test_stop_after_sync(self, mock_sleep, mock_thread):
        synchronizer = self._create_synchronizer()
        mock_thread_instance = mock.Mock()
        mock_thread.return_value = mock_thread_instance

        synchronizer.sync()
        synchronizer.stop()

        # Verify thread join was called
        mock_thread_instance.join.assert_called_once()

    def test_stop_without_sync(self):
        synchronizer = self._create_synchronizer()

        # Should not raise an exception
        synchronizer.stop()

    def test_single_level_inheritance_with_requirements(self):
        class DummySynchronizer(db_sync.BaseOvnDbSynchronizer):
            _required_mechanism_drivers = ['driver1', 'driver2']
            _required_service_plugins = ['plugin1']
            _required_ml2_ext_drivers = ['ext_driver1']

            def do_sync(self):
                pass

        self.assertEqual(['driver1', 'driver2'],
                         sorted(DummySynchronizer.
                                get_required_mechanism_drivers()))
        self.assertEqual(['plugin1'],
                         DummySynchronizer.get_required_service_plugins())
        self.assertEqual(['ext_driver1'],
                         DummySynchronizer.
                         get_required_ml2_extension_drivers())

    def test_multi_level_inheritance(self):
        class FirstLevelSynchronizer(db_sync.BaseOvnDbSynchronizer):
            _required_mechanism_drivers = ['driver1', 'driver2']
            _required_service_plugins = ['plugin1']
            _required_ml2_ext_drivers = ['ext_driver1']

            def do_sync(self):
                pass

        class SecondLevelSynchronizer(FirstLevelSynchronizer):
            # Include a duplicate 'driver1' from parent
            _required_mechanism_drivers = ['driver1', 'driver3']
            _required_service_plugins = ['plugin2', 'plugin3']
            _required_ml2_ext_drivers = ['ext_driver2']

            def do_sync(self):
                pass

        # Second level should have requirements from both levels
        # without duplicates
        mechanism_drivers = SecondLevelSynchronizer.\
            get_required_mechanism_drivers()
        self.assertEqual(3, len(mechanism_drivers))
        self.assertEqual(['driver1', 'driver2', 'driver3'],
                         sorted(mechanism_drivers))

        service_plugins = SecondLevelSynchronizer.\
            get_required_service_plugins()
        self.assertEqual(3, len(service_plugins))
        self.assertEqual(['plugin1', 'plugin2', 'plugin3'],
                         sorted(service_plugins))

        ml2_ext_drivers = SecondLevelSynchronizer.\
            get_required_ml2_extension_drivers()
        self.assertEqual(['ext_driver1', 'ext_driver2'],
                         sorted(ml2_ext_drivers))


class TestPluginConfiguration(base.BaseTestCase):

    _testplugin_config_file_opt = cfg.ListOpt(
        'testplugin_config_file', default=[])

    def test_register_additional_cli_opts(self):
        conf = cfg.ConfigOpts()

        class TestSynchronizer(db_sync.BaseOvnDbSynchronizer):
            @classmethod
            def register_additional_cli_opts(cls, conf):
                conf.register_cli_opts(
                    [TestPluginConfiguration._testplugin_config_file_opt])

            def do_sync(self):
                pass

        TestSynchronizer.register_additional_cli_opts(conf)
        conf(['--testplugin_config_file', '/tmp/test.conf'])
        self.assertEqual(['/tmp/test.conf'], conf.testplugin_config_file)

    def test_load_plugin_configuration(self):
        with tempfile.NamedTemporaryFile(mode='w', delete=False) as f:
            f.write('[testgroup]\n')
            f.write('test_value = plugin_value\n')
            config_path = f.name

        conf = cfg.ConfigOpts()
        conf.register_cli_opts([self._testplugin_config_file_opt])
        conf(['--testplugin_config_file', config_path])

        class TestSynchronizer(db_sync.BaseOvnDbSynchronizer):
            @classmethod
            def register_additional_cli_opts(cls, conf):
                conf.register_cli_opts(
                    [TestPluginConfiguration._testplugin_config_file_opt])

            @classmethod
            def register_plugin_config_opts(cls, conf):
                conf.register_opts(
                    [cfg.StrOpt('test_value', default='default')],
                    group='testgroup')

            @classmethod
            def get_plugin_config_files(cls, global_conf):
                return global_conf.testplugin_config_file

            def do_sync(self):
                pass

        plugin_conf = TestSynchronizer.load_plugin_configuration(conf)
        self.assertEqual('plugin_value', plugin_conf.testgroup.test_value)

    def test_load_plugin_configuration_returns_none_without_files(self):
        conf = cfg.ConfigOpts()

        class TestSynchronizer(db_sync.BaseOvnDbSynchronizer):
            def do_sync(self):
                pass

        self.assertIsNone(
            TestSynchronizer.load_plugin_configuration(conf))

    def test_plugin_conf_does_not_affect_global_conf(self):
        with tempfile.NamedTemporaryFile(mode='w', delete=False) as f:
            f.write('[testgroup]\n')
            f.write('test_value = plugin_value\n')
            config_path = f.name

        global_conf = cfg.ConfigOpts()
        global_conf.register_opts(
            [cfg.StrOpt('test_value', default='global_value')],
            group='testgroup')
        global_conf.register_cli_opts([self._testplugin_config_file_opt])
        global_conf(['--testplugin_config_file', config_path])

        class TestSynchronizer(db_sync.BaseOvnDbSynchronizer):
            @classmethod
            def register_plugin_config_opts(cls, conf):
                conf.register_opts(
                    [cfg.StrOpt('test_value', default='default')],
                    group='testgroup')

            @classmethod
            def get_plugin_config_files(cls, global_conf):
                return global_conf.testplugin_config_file

            def do_sync(self):
                pass

        plugin_conf = TestSynchronizer.load_plugin_configuration(global_conf)
        self.assertEqual('global_value', global_conf.testgroup.test_value)
        self.assertEqual('plugin_value', plugin_conf.testgroup.test_value)

    def test_synchronizer_stores_plugin_conf(self):
        plugin_conf = cfg.ConfigOpts()
        ovn_driver = mock.Mock()
        ovn_driver.nb_ovn = mock.Mock()
        ovn_driver.sb_ovn = mock.Mock()

        class TestSynchronizer(db_sync.BaseOvnDbSynchronizer):
            def do_sync(self):
                pass

        sync_obj = TestSynchronizer(
            mock.Mock(), ovn_driver, 'repair', plugin_conf=plugin_conf)
        self.assertIs(plugin_conf, sync_obj.plugin_conf)



# Copyright (c) 2025 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api import converters
from neutron_lib.api.definitions import l3
from neutron_lib.types import (
    ActionMap,
    ResourceAttributeMap,
    SubResourceAttributeMap,
)


ENABLE_SNAT = 'enable_snat'

ALIAS = 'router-enable-snat'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'Exposes the router "enable_snat" read-only configuration flag'
API_PREFIX = ''
DESCRIPTION = 'Exposes the router "enable_snat" read-only configuration flag'
UPDATED_TIMESTAMP = '2025-09-18T18:00:00-00:00'
RESOURCE_NAME = l3.ROUTER
COLLECTION_NAME = l3.ROUTERS
RESOURCE_ATTRIBUTE_MAP: ResourceAttributeMap = {
    COLLECTION_NAME: {
        ENABLE_SNAT: {'allow_post': False,
                      'allow_put': False,
                      'default': True,
                      'convert_to': converters.convert_to_boolean,
                      'required_by_policy': True,
                      'is_filter': True,
                      'is_sort_key': True,
                      'is_visible': True
                      },
    },
}
SUB_RESOURCE_ATTRIBUTE_MAP: SubResourceAttributeMap = {}
ACTION_MAP: ActionMap = {}
REQUIRED_EXTENSIONS = []
OPTIONAL_EXTENSIONS = []
ACTION_STATUS = {}



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from neutron_lib.types import (
    ActionMap,
    ResourceAttributeMap,
    SubResourceAttributeMap,
)

ALIAS = 'network-cascade-delete'
IS_SHIM_EXTENSION = True
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'Network Cascade Deletion Extension'
DESCRIPTION = ('Allows for cascade deletion of networks and all '
               'associated ports.')
UPDATED_TIMESTAMP = ''
RESOURCE_ATTRIBUTE_MAP: ResourceAttributeMap = {}
SUB_RESOURCE_ATTRIBUTE_MAP: SubResourceAttributeMap = {}
ACTION_MAP: ActionMap = {}
REQUIRED_EXTENSIONS = []
OPTIONAL_EXTENSIONS = []
ACTION_STATUS = {}



# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from neutron_lib.plugins.ml2 import api
from neutron_lib.tests import _base as base


class _MechanismDriver(api.MechanismDriver):
    def bind_port(self, context):
        return context

    def initialize(self):
        pass


class TestMechanismDriver(base.BaseTestCase):

    def test__supports_port_binding(self):
        self.assertTrue(_MechanismDriver()._supports_port_binding)

    def test_get_workers(self):
        self.assertEqual((), _MechanismDriver().get_workers())

    def test_filter_hosts_with_segment_access(self):
        dummy_token = ["X"]
        self.assertEqual(
            dummy_token,
            _MechanismDriver().filter_hosts_with_segment_access(
                dummy_token, dummy_token, dummy_token, dummy_token))

    def test_check_vlan_transparency_default(self):
        self.assertIsNone(_MechanismDriver().check_vlan_transparency(None))

    def test_check_vlan_qinq_default(self):
        self.assertIsNone(_MechanismDriver().check_vlan_qinq(None))


class _TypeDriver_Base:
    def get_type(self):
        pass

    def initialize(self):
        pass

    def is_partial_segment(self, segment):
        pass

    def validate_provider_segment(self, segment):
        pass

    def get_mtu(self, physical):
        pass

    def reserve_provider_segment(self, session, segment, filters=None):
        pass

    def allocate_tenant_segment(self, session, filters=None):
        pass

    def release_segment(self, session, segment):
        pass


class _TypeDriver_NoProject(_TypeDriver_Base, api.TypeDriver):
    pass


class _TypeDriver_WithProject(_TypeDriver_NoProject):

    def allocate_project_segment(self, session, filters=None):
        pass


class TestTypeDriver(base.BaseTestCase):

    def test_typedriver_noproject(self):
        drv = _TypeDriver_NoProject()
        drv.allocate_tenant_segment = mock.Mock()
        drv.allocate_project_segment('session', filters='filters')
        drv.allocate_tenant_segment.assert_called_once_with(
            'session', filters='filters')

    def test_typedriver_withproject(self):
        drv = _TypeDriver_WithProject()
        drv.allocate_tenant_segment = mock.Mock()
        drv.allocate_project_segment('session', filters='filters')
        drv.allocate_tenant_segment.assert_not_called()


class _ML2TypeDriver_NoProject(_TypeDriver_Base, api.ML2TypeDriver):

    def initialize_network_segment_range_support(self):
        pass

    def update_network_segment_range_allocations(self):
        pass


class _ML2TypeDriver_WithProject(_ML2TypeDriver_NoProject):

    def allocate_project_segment(self, context, filters=None):
        pass


class TestML2TypeDriver(base.BaseTestCase):

    def test_ml2typedriver_noproject(self):
        drv = _ML2TypeDriver_NoProject()
        drv.allocate_tenant_segment = mock.Mock()
        drv.allocate_project_segment('context', filters='filters')
        drv.allocate_tenant_segment.assert_called_once_with(
            'context', filters='filters')

    def test_ml2typedriver_withproject(self):
        drv = _ML2TypeDriver_WithProject()
        drv.allocate_tenant_segment = mock.Mock()
        drv.allocate_project_segment('context', filters='filters')
        drv.allocate_tenant_segment.assert_not_called()



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Context: context for security/db session."""

import collections
import copy

from oslo_context import context as oslo_context
from oslo_db.sqlalchemy import enginefacade
from oslo_log import log as logging
from oslo_utils import timeutils

from neutron_lib.db import api as db_api
from neutron_lib.policy import _engine as policy_engine

LOG = logging.getLogger(__name__)


class ContextBase(oslo_context.RequestContext):
    """Security context and request information.

    Represents the user taking a given action within the system.

    """

    def __init__(self, user_id=None, project_id=None, is_admin=None,
                 timestamp=None, project_name=None, user_name=None,
                 is_advsvc=None, is_service_role=None,
                 has_global_access=False, can_set_project_id=False, **kwargs):
        # NOTE(jamielennox): We maintain this argument order for tests that
        # pass arguments positionally.

        kwargs.setdefault('project_id', project_id)
        kwargs.setdefault('project_name', project_name)
        self._has_global_access = has_global_access
        self._can_set_project_id = can_set_project_id
        self._is_advsvc = is_advsvc
        self._is_service_role = is_service_role
        super().__init__(
            is_admin=is_admin, user_id=user_id, **kwargs)

        self.user_name = user_name
        self.timestamp = timestamp or timeutils.utcnow()

        if self._is_advsvc is not None:
            LOG.warning(
                "Argument 'is_advsvc' is deprecated since 2026.1 release "
                "(neutron-lib 3.23.0) and will be removed once support for "
                "the old RBAC API policies will be removed from Neutron. "
                "Please use argument 'is_service_role' instead.")
        else:
            # TODO(slaweq): remove this once is_advsvc will be removed and
            # we will rely on is_service_role only
            self._is_advsvc = (self.is_admin or
                               policy_engine.check_is_advsvc(self))

        if self._is_service_role is None:
            self._is_service_role = policy_engine.check_is_service_role(self)

        if self.is_admin is None:
            self.is_admin = policy_engine.check_is_admin(self)
        if not self._has_global_access:
            self._has_global_access = policy_engine.check_has_global_access(
                self)
        if not self._can_set_project_id:
            self._can_set_project_id = policy_engine.check_can_set_project_id(
                self)

    @property
    def is_service_role(self):
        # TODO(slaweq): we will not need to check ContextBase._is_advsvc once
        # we will get rid of the old API policies
        return self._is_service_role or self._is_advsvc

    @property
    def is_advsvc(self):
        # TODO(slaweq): this property should be removed once we will get rid
        # of old policy rules and only new, S-RBAC rules will be available as
        # then we will use ContextBase.is_service_role instead
        LOG.warning("Method 'is_advsvc' is deprecated since 2023.2 release "
                    "(neutron-lib 3.8.0) and will be removed once support for "
                    "the old RBAC API policies will be removed from Neutron. "
                    "Please use method 'is_service_role' instead.")
        return self.is_service_role

    @property
    def has_global_access(self):
        return self.is_admin or self._has_global_access

    @property
    def can_set_project_id(self):
        return (
            self.is_admin or
            self.is_service_role or
            self._can_set_project_id
        )

    def to_dict(self):
        context = super().to_dict()
        context.update({
            'user_id': self.user_id,
            'project_id': self.project_id,
            'timestamp': str(self.timestamp),
            'project_name': self.project_name,
            'user_name': self.user_name,
            'has_global_access': self.has_global_access,
            'can_set_project_id': self.can_set_project_id,
        })
        return context

    def to_policy_values(self):
        values = super().to_policy_values()
        values['is_admin'] = self.is_admin
        values['has_global_access'] = self.has_global_access
        values['can_set_project_id'] = self.can_set_project_id

        # NOTE(jamielennox): These are almost certainly unused and non-standard
        # but kept for backwards compatibility. Remove them in Pike
        # (oslo.context from Ocata release already issues deprecation warnings
        # for non-standard keys).
        values['user'] = self.user_id
        values['domain'] = self.domain_id
        values['user_domain'] = self.user_domain_id
        values['project_domain'] = self.project_domain_id
        values['project_name'] = self.project_name
        values['user_name'] = self.user_name

        return values

    @classmethod
    def from_dict(cls, values):
        cls_obj = super().from_dict(values)
        if values.get('timestamp'):
            cls_obj.timestamp = values['timestamp']
        cls_obj.user_id = values.get('user_id', values.get('user'))
        return cls_obj

    def elevated(self):
        """Return a version of this context with admin flag set."""
        context = copy.copy(self)
        context.is_admin = True

        context.roles = list(
            set(context.roles) | {'admin', 'member', 'reader'}
        )

        return context


@enginefacade.transaction_context_provider
class ContextBaseWithSession(ContextBase):
    pass


_TransactionConstraint = collections.namedtuple(
    '_TransactionConstraint', ['if_revision_match', 'resource', 'resource_id'])


class Context(ContextBaseWithSession):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._session = None
        self._txn_constraint = None

    @property
    def session(self):
        # TODO(ralonsoh): "Context.session" is provided by
        # "enginefacade.transaction_context_provider" when a new transaction,
        # READER or WRITER, is created. This property is just a temporary fix
        # for those transactions that are not executed inside a transaction.
        # By manually selecting the type of transaction we can speed up the
        # code because by default a writer session is always created, even
        # within read only operations.
        if hasattr(super(), 'session'):
            self._session = super().session
        if self._session is None:
            self._session = db_api.get_writer_session()
        return self._session

    def set_transaction_constraint(self, resource, resource_id, rev_number):
        """Set a revision constraint to enforce before resource_id is changed.

        :param resource: Collection name of resource (e.g. ports or networks)
        :param resource_id: The primary key ID of the individual resource that
                            should have its revision number matched before
                            allowing the transaction to proceed.
        :param rev_number: The revision_number that the resource should be at.
        """

        self._txn_constraint = _TransactionConstraint(
            if_revision_match=rev_number, resource=resource,
            resource_id=resource_id)

    def clear_transaction_constraint(self):
        self._txn_constraint = None

    def get_transaction_constraint(self):
        return self._txn_constraint


def get_admin_context():
    # NOTE(slaweq): elevated() method will set is_admin=True but setting it
    # explicity here will avoid checking in policy rules if is_admin should be
    # set to True or not
    return Context(user_id=None,
                   project_id=None,
                   is_admin=True,
                   overwrite=False).elevated()


def get_admin_context_without_session():
    # NOTE(slaweq): elevated() method will set is_admin=True but setting it
    # explicity here will avoid checking in policy rules if is_admin should be
    # set to True or not
    return ContextBase(user_id=None, project_id=None, is_admin=True).elevated()



# Copyright (c) 2026 Red Hat Inc.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from neutron_lib.api import converters
from neutron_lib.api.definitions import network
from neutron_lib.api.definitions import port
from neutron_lib.services.pvlan import constants as pvlan_const
from neutron_lib.types import (
    ActionMap,
    ResourceAttributeMap,
    SubResourceAttributeMap,
)


ALIAS = 'pvlan'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'Private VLAN'
DESCRIPTION = 'The Private VLAN semantics extension.'
UPDATED_TIMESTAMP = '2026-02-03T10:00:00-00:00'

RESOURCE_ATTRIBUTE_MAP: ResourceAttributeMap = {
    network.COLLECTION_NAME: {
        pvlan_const.PVLAN: {
            'allow_post': True,
            'allow_put': True,
            'is_filter': True,
            'is_visible': True,
            'default': False,
            'validate': {'type:boolean': None},
            'convert_to': converters.convert_to_boolean}
    },
    port.COLLECTION_NAME: {
        pvlan_const.PVLAN_TYPE: {
            'allow_post': True,
            'allow_put': True,
            'is_filter': True,
            'default': None,
            'is_visible': True,
            'validate': {'type:values': (*pvlan_const.PVLAN_TYPES, None)}
        },
        pvlan_const.PVLAN_COMMUNITY: {
            'allow_post': True,
            'allow_put': True,
            'is_filter': True,
            'is_visible': True,
            'default': None,
            'validate': {
                'type:regex_or_none': pvlan_const.COMMUNITY_NAME_REGEX}
        }
    }
}


SUB_RESOURCE_ATTRIBUTE_MAP: SubResourceAttributeMap = {}
ACTION_MAP: ActionMap = {}

ACTION_STATUS = {}
REQUIRED_EXTENSIONS = []
OPTIONAL_EXTENSIONS = []



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from neutron_lib.api.validators import dns
from neutron_lib.db import constants as db_constants
from neutron_lib.tests import _base as base


class TestDnsValidators(base.BaseTestCase):

    @mock.patch('oslo_config.cfg.CONF')
    def test_validate_dns_name(self, CONF):
        CONF.dns_domain = ''
        msg = dns.validate_dns_name('')
        self.assertIsNone(msg)

        CONF.dns_domain = 'example.org.'
        dns_name = 'host'
        msg = dns.validate_dns_name(dns_name)
        self.assertIsNone(msg)

        invalid_data = 'A' * 256
        max_len = 255
        expected = (f"'{invalid_data}' not a valid PQDN or FQDN. Reason: "
                    f"'{invalid_data}' exceeds the {max_len} character FQDN "
                    "limit")
        msg = dns.validate_dns_name(invalid_data, max_len)
        self.assertEqual(expected, msg)

        invalid_data = '.hostname'
        expected = (f"'{invalid_data}' not a valid PQDN or FQDN. Reason: "
                    "Encountered an empty component")
        msg = dns.validate_dns_name(invalid_data)
        self.assertEqual(expected, msg)

        invalid_data = 'hostname-'
        expected = (f"'{invalid_data}' not a valid PQDN or FQDN. Reason: "
                    f"Name '{invalid_data}' must not start or end with a "
                    "hyphen")
        msg = dns.validate_dns_name(invalid_data)
        self.assertEqual(expected, msg)

        invalid_data = 'hostname@host'
        expected = (f"'{invalid_data}' not a valid PQDN or FQDN. Reason: "
                    f"Name '{invalid_data}' must be 1-63 characters long, "
                    "each of which can only be alphanumeric or a "
                    "hyphen")
        msg = dns.validate_dns_name(invalid_data)
        self.assertEqual(expected, msg)
        invalid_suffix = '1234'
        invalid_data = 'hostname.' + invalid_suffix
        expected = (f"'{invalid_data}' not a valid PQDN or FQDN. Reason: "
                    f"TLD '{invalid_suffix}' must not be all "
                    "numeric")
        msg = dns.validate_dns_name(invalid_data)
        self.assertEqual(expected, msg)

        # len(dns_name + dns_domain) > 255
        invalid_domain = 'A' * 250 + '.org.'
        CONF.dns_domain = invalid_domain
        dns_name = 'hostname'
        expected = ("The dns_name passed is a PQDN and its size is "
                    f"'{len(dns_name)}'. The dns_domain option in "
                    f"neutron.conf is set to {invalid_domain}, with a "
                    f"length of '{len(invalid_domain) + 1}'. When the two are "
                    "concatenated to form a FQDN (with a '.' at the end), "
                    "the resulting length exceeds the maximum size "
                    f"of '{db_constants.FQDN_FIELD_SIZE}'"
                    )
        msg = dns.validate_dns_name(dns_name)
        self.assertEqual(expected, msg)

        dns_name = 'host.'
        dns_domain = 'example.com.'
        CONF.dns_domain = dns_domain
        expected = ("The dns_name passed is a FQDN. Its higher level labels "
                    "must be equal to the dns_domain option in neutron.conf, "
                    f"that has been set to '{dns_domain}'. It must also "
                    "include one or more valid DNS labels to the left "
                    f"of '{dns_domain}'")
        msg = dns.validate_dns_name(dns_name)
        self.assertEqual(expected, msg)

    def test_validate_fip_dns_name(self):
        # Don't run tests duplicated to validate_dns_name()

        msg = dns.validate_fip_dns_name('')
        self.assertIsNone(msg)

        msg = dns.validate_fip_dns_name('host')
        self.assertIsNone(msg)

        invalid_data = 1234
        expected = f"'{invalid_data}' is not a valid string"
        msg = dns.validate_fip_dns_name(invalid_data)
        self.assertEqual(expected, msg)

        invalid_data = 'host.'
        expected = (f"'{invalid_data}' is a FQDN. It should be a relative "
                    "domain name")
        msg = dns.validate_fip_dns_name(invalid_data)
        self.assertEqual(expected, msg)

        length = 10
        invalid_data = 'a' * length
        max_len = 12
        expected = (f"'{invalid_data}' contains {length} characters. Adding a "
                    "domain name will cause it to exceed the maximum length "
                    f"of a FQDN of '{max_len}'")
        msg = dns.validate_fip_dns_name(invalid_data, max_len)
        self.assertEqual(expected, msg)

    def test_validate_dns_domain(self):
        # Don't run tests duplicated to validate_dns_name()

        msg = dns.validate_dns_domain('')
        self.assertIsNone(msg)

        msg = dns.validate_dns_domain('example.com.')
        self.assertIsNone(msg)

        invalid_data = 1234
        expected = f"'{invalid_data}' is not a valid string"
        msg = dns.validate_dns_domain(invalid_data)
        self.assertEqual(expected, msg)

        invalid_data = 'example.com'
        expected = f"'{invalid_data}' is not a FQDN"
        msg = dns.validate_dns_domain(invalid_data)
        self.assertEqual(expected, msg)

        length = 9
        invalid_data = 'a' * length + '.'
        max_len = 11
        expected = (f"'{invalid_data}' contains {length + 1} characters. "
                    "Adding a sub-domain will cause it to exceed the maximum "
                    f"length of a FQDN of '{max_len}'")
        msg = dns.validate_dns_domain(invalid_data, max_len)
        self.assertEqual(expected, msg)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
# neutron documentation build configuration file, created by
# sphinx-quickstart on Sat May  1 15:17:47 2010.
#
# This file is execfile()d with the current directory set to
# its containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import os
import sys

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
sys.path.insert(0, os.path.abspath('../../'))
sys.path.insert(0, os.path.abspath('../'))
sys.path.insert(0, os.path.abspath('./'))

# -- General configuration ----------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    'os_api_ref',
    'openstackdocstheme',
]

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = 'Networking API Reference'
copyright = '2010-present, OpenStack Foundation'

release = ''
version = ''

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = False

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'native'

# -- Options for HTML output --------------------------------------------------

html_theme = 'openstackdocs'
html_theme_options = {
    'sidebar_mode': 'toc',
}

# -- Options for LaTeX output -------------------------------------------------

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto/manual]).
latex_documents = [
    ('index', 'Neutron.tex', 'OpenStack Networking API Documentation',
     'OpenStack Foundation', 'manual'),
]

# -- Options for linkcheck builder --------------------------------------------

linkcheck_anchors_ignore = [
    # skip gerrit anchors
    '\\/q\\/.*',
    'q\\,.*',
    '\\/c\\/.*'
]

# -- Options for openstackdocstheme -------------------------------------------

# openstackdocstheme options
openstackdocs_repo_name = 'openstack/neutron-lib'
openstackdocs_bug_project = 'neutron'
openstackdocs_bug_tag = 'api-ref'
openstackdocs_auto_name = False



# Copyright (c) 2017 OVH SAS
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import qos as qos_apidef
from neutron_lib.types import (
    ActionMap,
    ResourceAttributeMap,
    SubResourceAttributeMap,
)


DRIVERS = 'drivers'

ALIAS = 'qos-rule-type-details'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'Details of QoS rule types'
API_PREFIX = ''
DESCRIPTION = ("Expose details about QoS rule types supported by loaded "
               "backend drivers")
UPDATED_TIMESTAMP = '2017-06-22T10:00:00-00:00'
RESOURCE_ATTRIBUTE_MAP: ResourceAttributeMap = {
    qos_apidef.RULE_TYPES: {
        DRIVERS: {
            'allow_post': False,
            'allow_put': False,
            'is_visible': True
        }
    }
}
SUB_RESOURCE_ATTRIBUTE_MAP: SubResourceAttributeMap = {}
ACTION_MAP: ActionMap = {}
REQUIRED_EXTENSIONS = [qos_apidef.ALIAS]
OPTIONAL_EXTENSIONS = []
ACTION_STATUS = {}
